@@ -4,7 +4,7 @@ not_cran <- identical(Sys.getenv("NOT_CRAN"), "true")
 ## CAREFUL: update_gauss_disp = TRUE only for models stored *before*
 ##  var -> SD reparameterization
 ## (store API information in model object?)
-file_ok <- gt_load("test_data/models.rda", update_gauss_disp = TRUE)
+file_ok <- gt_load("test_data/models.rda")
 if (!file_ok) {
     make_ex <- system.file("test_data", "make_ex.R", package="glmmTMB", mustWork = TRUE)
     cat("running fits to build test examples ...\n")
